import colander
import requests
from colander import Invalid, null
from deform.widget import CheckedInputWidget


@colander.deferred
def deferred_recaptcha_widget(node, kw):
    request = kw['request']

    class RecaptchaWidget(CheckedInputWidget):
        template = 'recaptcha_widget'
        readonly_template = 'recaptcha_widget_readonly'
        requirements = ()
        url = "https://www.google.com/recaptcha/api/siteverify"
        headers = {'Content-type': 'text/plain'}

        def serialize(self, field, cstruct, readonly=False):
            if cstruct in (null, None):
                cstruct = ''
            # confirm = getattr(field, 'confirm', '')
            template = readonly and self.readonly_template or self.template
            settings = self.request.registry.settings
            public_key = settings['pyramid_recaptcha.public_key']
            return field.renderer(template,
                                  field=field,
                                  cstruct=cstruct,
                                  public_key=public_key)

        def deserialize(self, field, pstruct):
            if pstruct is null:
                return null
            response = pstruct.get('g-recaptcha-response') or ''
            if not response:
                raise Invalid(field.schema, 'No input')
            settings = self.request.registry.settings
            privatekey = settings['pyramid_recaptcha.private_key']
            remoteip = self.request.remote_addr
            request = requests.post(self.url, data={'secret': privatekey,
                                                    'response': response,
                                                    'remoteip': remoteip})
            if request.status_code != 200:
                raise Invalid(field.schema,
                              "There was an error talking to the recaptcha \
                              server ({0})".format(request.status_code))
            try:
                response = request.json()
            except ValueError as e:
                raise Invalid(field.schema,
                              "There was an error talking to the recaptcha \
                              server ({0})".format(e))
            valid = response['success']
            if valid is not True:
                # TODO: Should provide more meaningful error messages
                raise Invalid(field.schema, "Incorrect solution")
            return 'Verified'

    return RecaptchaWidget(request=request)



from pkg_resources import resource_filename
from deform import Form
from .recaptcha import deferred_recaptcha_widget  # noqa


def add_search_path():
    loader = Form.default_renderer.loader
    loader.search_path = (resource_filename('pyramid_recaptcha', 'templates'),)\
        + loader.search_path


def includeme(config):
    add_search_path()



import os

from setuptools import find_packages
from setuptools import setup

project = 'pyramid_recaptcha'
version = '1.0.2'

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
CHANGES = open(os.path.join(here, 'CHANGES.rst')).read()

requires = [
    'deform',
    'colander',
    'requests',
    'pyramid']

desc = 'A pyramid/deform widget implementing googles recaptcha widget.'

setup(
    name=project,
    version=version,
    description=desc,
    long_description=README + '\n\n' + CHANGES,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Pylons',
        'Framework :: Pyramid',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: User Interfaces'],
    keywords='pyramid deform recaptcha captcha',
    author='Jon Pentland, PretaGov Ltd',
    author_email='jon.pentland@pretagov.co.uk',
    url='https://github.com/pretagov/pyramid_recaptcha.git',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    install_requires=requires,
    extras_require={})
